<?php
/***********************************************************************
 * Module:  Controller.php
 * Author:  vallance
 * Purpose: Defines the Class Controller
 ***********************************************************************/


class Controller {
	
	const KERNEL_DIR = 'fusionSrc/kernelStuff/';
	const PLUGIN_DIR = 'fusionSrc/pluginStuff/';
	
   	private $argArray = array();
	
	/** 
	 * Default Controller constructor.
	 * Call method executeKernelFlowStack and executePluginFlowStack.
	 * 
	 * @param $argArray 
	 */
	public function __construct( $argArray) {
		
		$this->argArray = $argArray;
		
		if($this->executeKernelFlowStack()){
			
			$this->executePluginFlowStack();
		
		}
		
	}
	 
	/**
	 * Scan kernel class directory and execute each control statement.
	 * If a class control return true, it will execute the method suggested
	 * by getControlMethod method and exit. Else, the executePluginFlowStack
	 * will be execute.
	 * 
	 * @return boolean
	 */
	public function executeKernelFlowStack() {
		
		$dir = dir(self::KERNEL_DIR);
		
		while ($entry = $dir->read()){
			if(is_file(self::KERNEL_DIR.$entry) && preg_match("/\.php$/", $entry)){
				
				$kernelStackName = substr($entry, 0, strlen($entry) - strlen(".php"));
				$kernelStack = new $kernelStackName();
				
				if($kernelStack->control($this->argArray)){
					$controlMethod = $kernelStack->getControlMethod();
					$kernelStack->$controlMethod();
					unset($kernelStack);
					return false;
				}
				
				unset($kernelStack);
			}
		}
		
		return true;
	}
	 
	/**
	 * Scan plugin class directoryand execute each control statement.
	 * If a class control return true, it will execute the method suggested
	 * by getControlMethod method and exit.
	 * 
	 * @return 0
	 */
	public function executePluginFlowStack() {
		
		$dir = dir(self::PLUGIN_DIR);
		
		while ($entry = $dir->read()){
			if(is_file(self::PLUGIN_DIR.$entry) && preg_match("/\.php$/", $entry)){
				
				$pluginStackName = substr($entry, 0, strlen($entry) - strlen(".php"));
				$pluginStack = new $pluginStackName();
				
				if($pluginStack->control($this->argArray)){
					$controlMethod = $pluginStack->getControlMethod();
					$pluginStack->$controlMethod();
					unset($pluginStack);
					return 0;
				}
				
				unset($pluginStack);
			}
		}
		
		return 0;
	}

}