<?php
/***********************************************************************
 * Module:  AJSFinder.php
 * Author:  vallance
 * Purpose: Defines the Class AJSFinder
 ***********************************************************************/

class AJSFinder {

	private $curentConfig;
	private $parser;
	private $ajsFiles;
	
	private $ajsArray = array();
	private $fileManager;
	
	private $color;
	
	public function __construct() {
		
		$this->curentConfig = new ConfigObject();
		
		$this->curentConfig->autoLoad();
		
		$this->color = new Colors();
		
	}
	
	/**
	 * Parse and compile all ajs files.
	 * 
	 * @param AJSParser $ajsParse
	 */
	public function execute(AJSParser $ajsParse) {
		
		$this->parser = $ajsParse;
		
		$this->ajsFiles = $this->getAjsFileName();
		$this->fileManager = new FileManager();
		
		foreach ($this->ajsFiles as $ajsFile) {
			
			$this->ajsArray[] = $this->parser->parseAJS($ajsFile, $this);
			
		}
			
		foreach ($this->ajsArray as $ajs) {
			$ajs->compile($this->fileManager);
		}
	}
	
	/**
	 * Parse and compile all ajs files ans wait for change.
	 * Create new ajs file if created.
	 * 
	 * @param AJSParser $ajsParse
	 * @param number $sleep
	 * @param string $indicator
	 */
	public function watching(AJSParser $ajsParse, $sleep = 1, $indicator = "") {
		
		$this->parser = $ajsParse;
		$this->ajsFiles = $this->getAjsFileName();
		$this->fileManager = new FileManager();
		
		foreach ($this->ajsFiles as $ajsFile) {
			
			$this->ajsArray[] = $this->parser->parseAJS($ajsFile, $this);
			
			foreach ($this->ajsArray as $ajs) {
				$ajs->compile($this->fileManager);
			}
			
		}
		
		$loopCount = 0;
		$displayBool = true;
		while(true){
			
			if($displayBool){
				$this->color->echoColor ( "--- waiting for change ---", "black", "light_gray" );
				echo "\n";
				$displayBool = false;
			}
			
			if($indicator != ""){
				$loopCount++;
				echo str_replace("%", "\n", str_replace("$", $loopCount, $indicator));
			}
			sleep ( $sleep );

			if(count($this->getAjsFileName()) != count($this->ajsFiles)){
				$this->ajsFiles = $this->getAjsFileName();
				foreach ($this->ajsFiles as $ajsFile) {
					$this->ajsArray[] = $this->parser->parseAJS($ajsFile, $this);
					foreach ($this->ajsArray as $ajs) {
						$ajs->compile($this->fileManager);
					}
				}
			}
			
			foreach ($this->ajsArray as $ajs) {
				if($ajs->checkChange($this->fileManager)){
					$ajs->compile($this->fileManager);
					$displayBool = true;
				}
			}
		}
		
	}

	/**
	 * Return curent configuration from config file.
	 * 
	 * @return ConfigObject
	 */
	public function getCurentConfig() {
		return $this->curentConfig;
	}
	
	/**
	 * Return all of existing ajs source files.
	 * 
	 * @param string $fileName
	 * @return array string 
	 */
	public function getAjsFileName($fileName = null) {
		
		$files = array();
		
		foreach ($this->curentConfig->getAjsDirectory() as $ajsDir){
			$ajsDir = trim($ajsDir);
			if(!preg_match('/\/$/', $ajsDir)){
				$ajsDir .= '/';
			}
			
			foreach ($this->getDirAjsFile($ajsDir) as $file) {
				if($fileName !== null && substr($file, strlen($ajsDir)) == $fileName){
					$files[] = $file;
				}else if($fileName === null){
					$files[] = $file;
				}
			}
			
		}
		
		return $files;
		
	}
	
	/**
	 * Return all existing ajs source file name from existing ajs directory.
	 * 
	 * @param string $dirPath
	 * @return array
	 */
	public function getDirAjsFile($dirPath) {
		
		$files = array();
		
		if(is_dir($dirPath)){
			$rep = dir($dirPath);
			while(($file = $rep->read()) !== false){
				
				if(preg_match('/.ajs$/', $file)){
					$files[] = $dirPath.$file;
				}
				
			}
		}
		
		return $files;
		
	}
	
}