<?php
/***********************************************************************
 * Module:  AJSParser.php
 * Author:  vallance
 * Purpose: Defines the Class AJSParser
 ***********************************************************************/

class AJSParser {

	private $ajsConfig;
	private $color;
	
	private $finder;
	
	public function __construct() {
		$this->color = new Colors();
	}
	
	/**
	 * Allow an AjsObject to reparse from it ajs file.
	 * 
	 * @param string $filePath
	 * @return AjsObject
	 */
	public function reparse($filePath) {
		return $this->parseAJS($filePath, $this->finder);
	}
	
	/**
	 * Parse an ajs file and return AjsObject.
	 * 
	 * @param string $filePath
	 * @param AJSFinder $finder
	 * @return AjsObject
	 */
	public function parseAJS($filePath, AJSFinder $finder) {
		
		$this->finder = $finder;
		$this->cloneConfig($finder->getCurentConfig()->getConfig());

		$filesArray = array();
		$shellInside = "";
		$outputFile = null;
		
		if(file_exists($filePath)){
			
			$fm = fopen('php://memory', 'r+');
			fputs($fm, file_get_contents($filePath));
			rewind($fm);
			
			$lineCount = 0;
			
			while ( $line = fgets ( $fm ) ) {
				$lineCount ++;
				
				$line = preg_replace ( "/\\n/", "", $line );
			
				if ( ! preg_match ( "/^# /", $line ) ) {
					if ( ! preg_match ( "/^@/", $line ) && trim($line) != "" ) {
						if($this->isFirst($line, $filesArray)){
							$filesArray[] = [$line, 
											$this->ajsConfig['output'], 
											$this->ajsConfig['lineBreak'],
											$this->ajsConfig['verbose'],
											$this->ajsConfig['minify']
							];
						}
					} elseif ( preg_match ( "/^@extend /", $line ) ) {
						$fileExtend = $finder->getAjsFileName(substr($line, strlen("@extend ")));
						if(count($fileExtend)){
							$arr = $this->parseAJS($fileExtend[0], $finder);
							
							for($i = 0;$i < $arr->count();$i++){
								if($arr->count() && $this->isFirst($arr->getFiles($i), $filesArray)){
									$filesArray[] = [$arr->getFiles($i), 
													$this->ajsConfig['output'], 
													$this->ajsConfig['lineBreak'],
													$this->ajsConfig['verbose'],
													$this->ajsConfig['minify']
									];
								}
							}
						}
					} else if (preg_match("/^@shell /", $line)){
						$shellInside .= substr($line, strlen("@shell "));
					} else if (preg_match("/^@shell{\$/", $line)){
						while ($line = fgets ( $fm )) {
							$lineCount ++;
							
							$line = preg_replace ( "/\\n/", "", $line );
							if(preg_match("/^}@$/", $line)){
								break 1;
							}else{
								$shellInside .= $line;
							}
						}
					} else if(preg_match("/^@config /", $line)){
						$outConfig = array();
						preg_match("/^@config (.+) (.+)$/", $line, $outConfig);
						
						if(count($outConfig) == 3){
							if(isset($this->ajsConfig[$outConfig[1]])){
								switch ($outConfig[1]){
									case "output":
										$outConfig[2] = (string)$outConfig[2];
										break;
									case "lineBreak":
										$outConfig[2] = (int)$outConfig[2];
										break;
									case "verbose":
										$outConfig[2] = (bool)$outConfig[2];
										break;
									case "minify":
										$outConfig[2] = (bool)$outConfig[2];
										break;
								}
								$this->ajsConfig[$outConfig[1]] = $outConfig[2];
								
							}else{
								echo $this->color->getColoredString("AJS error : ", 'red')."config option error into $filePath at line $lineCount \n";
							}
						}else{
							echo $this->color->getColoredString("AJS error : ", 'red')."config line error into $filePath at line $lineCount \n";
						}
					}elseif(preg_match("/^@outputFile /", $line)){
						$outputFile = substr($line, strlen("@outputFile "));
					}
				}
				
				
			}
			
			fclose ( $fm );
		}else{
			
		}
		
		return new AjsObject($filesArray, $shellInside, $this->ajsConfig, $filePath, $this, ($outputFile !== null ?$outputFile:null));
		
	}
	
	/**
	 * Clone current configuration for safe change.
	 * 
	 * @param array $config
	 */
	public function cloneConfig($config) {
		
		$this->ajsConfig = array();
		
		foreach ($config as $key=>$value) {
			$this->ajsConfig[$key] = $value;
		}
		
	}
	
	/**
	 * Check if specified file is the first occurence into the ajs file.
	 * 
	 * @param string $jsFileName
	 * @param array $fileArray
	 * @throws Exception
	 * @return boolean
	 */
	public function isFirst($jsFileName, $fileArray) {
		
		if(is_array($fileArray)){
			foreach ($fileArray as $arr) {
				if($arr[0] == $jsFileName){
					return false;
				}
			}
			
			return true;
		}else{
			echo $this->color->getColoredString("Process error.", "red")."\n";
			throw new Exception("Process error.", 500);
		}
		
	}
	
}