<?php
/***********************************************************************
 * Module:  AjsObject.php
 * Author:  vallance
 * Purpose: Defines the Class AjsObject
 ***********************************************************************/

class AjsObject {

	const FILE_NAME = 0;
	const OUTPUT = 1;
	const LINE_BREAK = 2;
	const VERBOSE = 3;
	const MINIFY = 4;
	const OUTPUT_NAME = 5;
	const LENGTH = 6;
	
	private $selfName;
	
	private $files = array();
	private $shell = "";
	private $config;
	private $length;
	
	private $color;
	private $outputFile = null;
	
	private $path;
	private $content;
	
	private $parser;
	
	public function __construct($files, $shell, $config, $selfName, AJSParser $parser, $outputFile = null) {
		
		$this->files = $files;
		$this->shell = $shell;
		$this->config = $config;
		$this->path = $selfName;
		$this->selfName = substr($selfName, (strrpos($selfName, '/')+1), -3)."js";
		
		$this->color = new Colors();
		
		$this->outputFile = $outputFile;
		$this->content = file_get_contents($this->path);
		
		$this->parser = $parser;
	}
	
	/**
	 * Check if a change exist into source file content or ajs file.
	 * Call reparse AJSParser method if a change exist into ajs file.
	 * 
	 * @param FileManager $fileManager
	 * @return boolean
	 */
	public function checkChange(FileManager $fileManager) {

		$fileContent = "";

		for ($i = 0; $i < $this->count(); $i++) {
			if(($curentFile = $fileManager->getInputFile($this->files[$i][self::FILE_NAME],
					$this->config['jsDirectory'])) !== null){
				$fileContent .= $curentFile;
			}
		}
		
		if($this->length != strlen($fileContent)){
			return true;
		}else{
			
			if($this->content != file_get_contents($this->path)){
				$me = $this->parser->reparse($this->path);
				
				$this->config = $me->getConfig();
				$this->content = $me->getContent();
				$this->files = $me->getAllFiles();
				$this->outputFile = $me->getOutputFile();
				$this->shell = $me->getShell();
				
				return true;
			}
			
			return false;
		}
	}

	/**
	 * Return source ajs file name.
	 * 
	 * @return string
	 */
	public function getSelfName() {
		return $this->selfName;
	}

	/**
	 * Return the result config.
	 */
	public function getConfig() {
		return $this->config;
	}

	/**
	 * Return result file length.
	 * 
	 * @return number
	 */
	public function getLength() {
		return $this->length;
	}

	/**
	 * Return the content Colors object.
	 * 
	 * @return Colors
	 */
	public function getColor() {
		return $this->color;
	}

	/**
	 * Return the output file name.
	 * 
	 * @return string
	 */
	public function getOutputFile() {
		return $this->outputFile;
	}

	/**
	 * return ajs file path.
	 */
	public function getPath() {
		return $this->path;
	}

	/**
	 * Return self ajs file content.
	 * 
	 * @return string
	 */
	public function getContent() {
		return $this->content;
	}

	/**
	 * Return AJSParser
	 * 
	 * @return AJSParser
	 */
	public function getParser() {
		return $this->parser;
	}
	
	/**
	 * return an indexed js source file name.
	 * 
	 * @param number $index
	 */
	public function getFiles($index) {
		return $this->files[$index][0];
	}
	
	/**
	 * Return all of the js source file name that compose the result file.
	 * 
	 * @return array
	 */
	public function getAllFiles() {
		return $this->files;
	}

	/**
	 * Return the js source file count that compose the result file.
	 * 
	 * @return number
	 */
	public function count() {
		return count($this->files);
	}
	
	/**
	 * Return js source include into ajs file.
	 * 
	 * @return string
	 */
	public function getShell() {
		return $this->shell;
	}
	
	/**
	 * Compile js result from js source files.
	 * 
	 * @param FileManager $fileManager
	 */
	public function compile(FileManager $fileManager) {
		
		$config = array();
		$fileContent = "";
		
		if($this->config['verbose']){
			echo $this->color->getColoredString("build file ", 'green').$this->selfName."\n";
		}
			
		for ($i = 0; $i < $this->count(); $i++) {

			if(($curentFile = $fileManager->getInputFile($this->files[$i][self::FILE_NAME], 
					$this->config['jsDirectory'])) !== null){
			
				$config[self::OUTPUT] = $this->files[$i][self::OUTPUT];
				$config[self::LINE_BREAK] = $this->files[$i][self::LINE_BREAK];
				$config[self::MINIFY] = $this->files[$i][self::MINIFY];
				$config[self::VERBOSE] = $this->files[$i][self::VERBOSE];
				
				$fileContent .= $curentFile;
			
				$this->files[$i][self::LENGTH] = strlen($fileContent);

				if($config[self::VERBOSE]){
					echo $this->color->getColoredString("\t from file ", 'blue').$this->files[$i][self::FILE_NAME]."\n";
				}
				
			}else{
				
				echo $this->color->getColoredString("Error : ", "red")."file ".$this->files[$i][self::FILE_NAME]." cant be read or inexist.\n";
				
			}
		}
		
		$fileContent .= $this->shell;
		
		$this->length = strlen($fileContent);
		
		if (!preg_match('/\/$/', $config[self::OUTPUT])){
			$config[self::OUTPUT] .= '/';
		}

		if($config[self::VERBOSE]){
			if($this->outputFile === null){
				echo $this->color->getColoredString("\t\t into output file ", 'green').$config[self::OUTPUT].$this->selfName."\n";
			}else{
				echo $this->color->getColoredString("\t\t into output file ", 'green').$config[self::OUTPUT].$this->outputFile."\n";
			}
		}
		
		if($config[self::MINIFY]){

			if($config[self::VERBOSE]){
				echo $this->color->getColoredString("\t minify output file ", 'blue')."\n";
			}
			
			file_put_contents($config[self::OUTPUT].'~'.$this->selfName, $fileContent);
			$cmd = "java -jar yuicompressor-2.4.8.jar --line-break " . $config[self::LINE_BREAK] . " " .
					$config[self::OUTPUT] . "~" . $this->selfName;
			$fileContent = shell_exec ( $cmd );
			unlink($config[self::OUTPUT].'~'.$this->selfName);

			if($config[self::VERBOSE]){
				echo $this->color->getColoredString("\t done ", 'green')."\n";
			}
			
		}

		if($this->outputFile === null){
			$fileManager->setOutputFile($this->selfName, $config[self::OUTPUT], $fileContent);
		}else{
			$fileManager->setOutputFile($this->outputFile, $config[self::OUTPUT], $fileContent);
		}
		
	}
	
	
}