<?php
/***********************************************************************
 * Module:  ConfigObject.php
 * Author:  vallance
 * Purpose: Defines the Class ConfigObject
 ***********************************************************************/

class ConfigObject implements FlowStackInterface {
	private $ajsDirectory = array();
	private $jsDirectory = array();
	private $output;
	private $lineBreak;
	private $verbose;
	private $minify;
	 
	public function __construct() {
	}
	 
	public function generate() {
		
		$configGenerator = new ConfigGenerator();
		$config = $configGenerator->generate();
		unset($configGenerator);
		
		$this->setAjsDirectory($config['ajs']);
		$this->setJsDirectory($config['js']);
		$this->setOutput($config['output']);
		$this->setVerbose(((strtoupper($config['verb']) == 'TRUE')?true:false));
		$this->setMinify(((strtoupper($config['min']) == 'TRUE')?true:false));
		$this->setLineBreak(($config['linebreak'] < 40)?40:$config['linebreak']);

		echo "\n\n-----\n".(new Colors())->getColoredString("Checking directory", 'light_red');
		foreach ($this->ajsDirectory as $directory) {
			if(!is_dir($directory)){
				echo "\n".(new Colors())->getColoredString("Generate directory ".$directory, 'red');
				mkdir($directory, 0777, true);
			}
		}
		foreach ($this->jsDirectory as $directory) {
			if(!is_dir($directory)){
				echo "\n".(new Colors())->getColoredString("Generate directory ".$directory, 'red');
				mkdir($directory, 0777, true);
			}
		}
		if(!is_dir($this->getOutput())){
			echo "\n".(new Colors())->getColoredString("Generate directory ".$this->getOutput(), 'red');
			mkdir($this->getOutput(), 0777, true);
		}
		echo "\n".(new Colors())->getColoredString("DONE", 'green')."\n-----\n";
		
		$XMLparser = new XMLParser($this);
		$XMLparser->create();
		unset($XMLparser);
		
		return false;
	}
	 
	/**
	 * Return the configuration file into an array.
	 * 
	 * @return array
	 */
	public function getConfig() {
		
		return array('ajsDirectory'=>$this->ajsDirectory,
						'jsDirectory'=>$this->jsDirectory,
						'output'=>$this->output,
						'lineBreak'=>$this->lineBreak,
						'verbose'=>$this->verbose,
						'minify'=>$this->minify
		);
	}
	 
	/**
	 * insert an ajs file directory into the configuration.
	 * 
	 *  @param newAjsDirectory 
	 */
	public function addAjsDirectory($newAjsDirectory) {
		
		$this->ajsDirectory[] = $newAjsDirectory;
	}
	 
	/**
	 * return an array that contain all of the ajs file directory for the configuration.
	 * 
	 * @return array
	 */
	public function getAjsDirectory() {
		return $this->ajsDirectory;
	}
	 
	/** 
	 * Set the ajs file directory array with a new array.
	 * 
	 * @param newAjsDirectory 
	 */
	public function setAjsDirectory($newAjsDirectory) {
		
		$this->ajsDirectory = $newAjsDirectory;
	}
	 
	/** 
	 * insert an javaScript source file directory into the configuration.
	 * 
	 * @param newJsDirectory 
	 */
	public function addJsDirectory($newJsDirectory) {
		
		$this->jsDirectory[] = $newJsDirectory;
	}
	 
	/**
	 * Return an array that contain all of the javaScript source file directory for the configuration.
	 * 
	 * @return array
	 */
	public function getJsDirectory() {
		return $this->jsDirectory;
	}
	 
	/** 
	 * Set the javaScript array with a new array.
	 * 
	 * @param newJsDirectory 
	 */
	public function setJsDirectory($newJsDirectory) {
		
		$this->jsDirectory = $newJsDirectory;
	}
	 
	/**
	 * Return the output directory.
	 * 
	 * @return string
	 */
	public function getOutput() {
		return $this->output;
	}
	 
	/**
	 * Set the output directory.
	 *  
	 * @param newOutput 
	 */
	public function setOutput($newOutput) {
		
		$this->output = $newOutput;
	}
	 
	/**
	 * Return the maximum line compilation length.
	 * 
	 * @return integer
	 */
	public function getLineBreak() {
		return $this->lineBreak;
	}
	 
	/**
	 * Set the maximum line compilation length.
	 * 
	 * @param newLineBreak 
	 */
	public function setLineBreak($newLineBreak) {
		
		$this->lineBreak = $newLineBreak;
	}
	 
	/**
	 * Return the verbose state of display.
	 * 
	 * @return boolean
	 */
	public function getVerbose() {
		return $this->verbose;
	}
	 
	/** 
	 * Set the verbose state of display.
	 * 
	 * @param newVerbose 
	 */
	public function setVerbose($newVerbose) {
		
		$this->verbose = $newVerbose;
	}
	 
	/**
	 * Return the minifying state of compilation protocol.
	 * 
	 * @return boolean
	 */
	public function getMinify() {
		return $this->minify;
	}
	 
	/** 
	 * Set the minifying state of compilation protocol.
	 * 
	 * @param newMinify 
	 */
	public function setMinify($newMinify) {
		
		$this->minify = $newMinify;
	}
	 
	/**
	 * Autoload the configuration from current directory configuration file.
	 * Return true on success.
	 * 
	 * @return boolean
	 */
	public function autoLoad() {
		if(file_exists(XMLParser::CONFIG_FILE_PATH)){
			$xmlconfig = simplexml_load_file(XMLParser::CONFIG_FILE_PATH);

			if(($config = $xmlconfig->children()) && $config->getName() == 'config'){
				foreach ($config->children() as $child) {
					
					switch($child->getName()){
						case 'ajs':
							foreach ($child->children() as $ajs) {
								$this->ajsDirectory[] = (string)$ajs;
							}
							break 1;
						case 'js':
							foreach ($child->children() as $js) {
								$this->jsDirectory[] = (string)$js;
							}
							break 1;
						case 'output':
							$this->output = (string)$child;
							break 1;
						case 'lineBreak':
							$this->lineBreak = (int)$child;
							break 1;
						case 'verbose':
							$this->verbose = (boolean)((string)$child ? true:false);
							break 1;
						case 'minify':
							$this->minify = (boolean)((string)$child ? true:false);
							break 1;
					}
					
				}
			}
			
			if(count($this->ajsDirectory) == 0 || count($this->jsDirectory) == 0 || empty($this->output)){
				return false;
			}
			
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Control if the call array arguments referer to the class usage.
	 * This class can be call by create:config with insensitive case.
	 *  
	 * @see FlowStackInterface::control()
	 */
	public function control( $argArray ) {
		
		if(isset($argArray[1]) && strtoupper($argArray[1]) == "CREATE:CONFIG"){
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Return the method name to call if the control method return true.
	 * 
	 * @see FlowStackInterface::getControlMethod()
	 */
	public function getControlMethod() {
		return 'generate';
	}

}