<?php
/***********************************************************************
 * Module:  HelpObject.php
 * Author:  vallance
 * Purpose: Defines the Class HelpObject
 ***********************************************************************/


class HelpObject implements FlowStackInterface {

	private $version;
	private $createConfig;
	private $update;
	private $watch;
	private $help;
	private $argArray = array();
	
	/**
	 * Default constructor of HelpObject. Define the help text for
	 * each elements.
	 */
	public function __construct(){
		
		$color = new Colors();
		$defaultColor = 'green';
		
		$this->version = $color->getColoredString("php fusion.php -v|--version\n", $defaultColor);
		$this->version .= "\tprint the current version of fusion.\n";
		
		$this->createConfig = $color->getColoredString("php fusion.php create:config\n", $defaultColor);
		$this->createConfig .= "\tlaunch an interactive script to create a configuration file.\n";
		$this->createConfig .= "\tvalues into [] are default value. Just press ENTER to use it.\n";
		
		$this->update = $color->getColoredString("php fusion.php update\n", $defaultColor);
		$this->update .= "\tupdate all js file.\n";
		
		$this->watch = $color->getColoredString("php fusion.php watch [waiting time] [loop indicator]\n", $defaultColor);
		$this->watch .= "\tlaunch the execution and wait for change. If change was detected on a file, this file is regenerate.\n";
		$this->watch .= "\tthis execution need a configuration file.\n";
		$this->watch .= "\t\twaiting time : time in seconds.\n";
		$this->watch .= "\t\tloop indicator : a string to write after a scan loop. % will write a line feed, $ will write loop count.\n";
		
		$this->help = $color->getColoredString("php fusion.php -h [function...]\n", $defaultColor);
		$this->help .= "\tget help about all of the functions or about a named function.\n";
		$this->help .= "\t\tfunction : \n";
		$this->help .= "\t\t\tversion\n";
		$this->help .= "\t\t\tcreate:config\n";
		$this->help .= "\t\t\tupdate\n";
		$this->help .= "\t\t\twatch\n";
		
		unset($color);
		
	}

	/**
	 * Control if the call array arguments referer to the class usage.
	 * This class can be call by -h or --help, with insensitive case.
	 *  
	 * @see FlowStackInterface::control()
	 */
	public function control( $argArray ) {
		
		if(isset($argArray[1]) && (strtoupper($argArray[1]) == "-H" || strtoupper($argArray[1]) == "--HELP")){
		
			$this->argArray = $argArray;
			
			return true;
			
		}else{
			return false;
		}
	}

	/**
	 * Return the method name to call if the control method return true.
	 * 
	 * @see FlowStackInterface::getControlMethod()
	 */
	public function getControlMethod() {
		return 'display';
	}
   
   /**
    * Display the wanted function helping text or the default
    * function choice if none is specify.
    * 
    * An error can be display if a specified function does'nt exist.
    */
   public function display() {
		if(count($this->argArray) > 2){
			
			$keyMethod = array('VERSION'=>'getVersion',
							'CREATE:CONFIG'=>'getCreateConfig',
							'UPDATE'=>'getUpdate',
							'WATCH'=>'getWatch'
			);
			
			for ($i = 2; $i < count($this->argArray); $i++) {
				if(isset($keyMethod[strtoupper($this->argArray[$i])])){
					echo $this->$keyMethod[strtoupper($this->argArray[$i])]()."\n";
				}else{
					echo (new Colors())->getColoredString("ERROR : ".$this->argArray[$i]." is not a valid function\n", 'red');
					echo $this->getHelp()."\n";
				}
			}
			
		}else{
			echo $this->getHelp()."\n";
		}
   }
   
   /**
    * return the version helping text.
    * 
    * @return string
    */
   public function getVersion() {
      return $this->version;
   }
   
   /**
    * return the create:config helping text.
    * 
    * @return string
    */
   public function getCreateConfig() {
      return $this->createConfig;
   }
   
   /**
    * return the update helping text.
    * 
    * @return string
    */
   public function getUpdate() {
      return $this->update;
   }
   
   /**
    * return the watch helping text.
    * 
    * @return string
    */
   public function getWatch() {
      return $this->watch;
   }
   
   /**
    * return the default helping text.
    * 
    * @return string
    */
   public function getHelp() {
      return $this->help;
   }

}